#ifndef JPLUS_H
#define JPLUS_H

#include <istream>
#include <vector>
#include <set>
#include "jengine.h"

class BasicBlock;

/**
 * A J+ script.
 */
class jplus: public jengine {
 private:
  BasicBlock* program;

 public:
  /** initialize J+ engine with no script */
  jplus();

  /** Load and pre-parse J+ script. 
   * @param script stream, containing the script to load. 
   * @return "true" upon success. */
  bool init(std::istream &script);

  /** Free allocated memory. */
  ~jplus();
  
  /** Set value of the specified variable inside J+ environment,
   *   mark all dependent variables "dirty". 
   * @param name name of the variable.
   * @param data array for the variable to refer to.
   * @return "true" upon success. */
  bool set(std::string name, jarray data);

  /** Perform all the necessary calculations to computate the
   *  specified variable, return its value in an array.
   * @param name name of variable to compute. 
   * @return the computed variable value. */
  jarray get(std::string name);

  /** Pull minimal set of J sentences for computing a given set of variables, 
   *  assuming everything is "dirty".
   * @param vars set of variable names to pull.
   * @return vector of J sentences for computing said variables.
   */
  std::vector<std::string> getProgram(std::set<std::string> vars);

  /** Pull all the sentences in J+ script.
   * This pulls all the sentences, assigning value of some variable.
   * @return vector of J sentences in J script. */
  std::vector<std::string> getProgram();

 protected:
  /** Called by the constructor before parsing the J+ script.
   *  Does nothing by default, but can be overridden to prepend
   *  an additional J+ stream with some definitions and/or define
   *  a few verbs and adverbs to be used in J+ program. */
  virtual void libInit();

 private:
  // copy disabled
  jplus(const jplus &);
  const jplus& operator=(const jplus&);
};

#endif
